pub struct ErrorObject {
pub code: Option<String>,
pub detail: Option<String>,
pub id: Option<String>,
pub source: Option<Box<ErrorObjectSource>>,
pub status: Option<String>,
}Fields
code: Option<String>application-specific error code
detail: Option<String>human-readable explanation specific to this occurrence of the problem
id: Option<String>unique identifier for this particular occurrence of the problem
source: Option<Box<ErrorObjectSource>>status: Option<String>HTTP status code applicable to this problem
Implementations
Sourceimpl ErrorObject
impl ErrorObject
pub fn new() -> ErrorObject